/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.db.server;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Connection;

import de.frankmuenster.jameica.orverw.OrdnerVerwaltung;
import de.frankmuenster.jameica.orverw.db.rmi.Version;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.jameica.plugin.AbstractPlugin;
import de.willuhn.jameica.plugin.Manifest;
import de.willuhn.jameica.plugin.PluginResources;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;
import de.willuhn.sql.version.UpdateProvider;
import de.willuhn.util.ApplicationException;
import de.willuhn.util.ProgressMonitor;

/**
 * The UpdateProvider that takes care about DB and other updates.
 *
 * @author Frank Mnster
 */
public class OrVerwUpdateProvider implements UpdateProvider {

    /**
     * Translation key for 'errorReadCurrentVersion'
     */
    private static final String TRANS_ERROR_READ_CURRENT_VERSION = "OrVerwUpdateProvider.errorReadCurrentVersion"; // NOPMD //$NON-NLS-1$

    /**
     * Die Version
     */
    private final transient Version version;

    /**
     * Ein SQL Verbindungsobjekt
     */
    private final transient Connection conn;

    /**
     * Das Plugin-Manifest
     */
    private final transient Manifest manifest;

    /**
     * Die Plugin Resoucen
     */
    private final transient PluginResources res;

    /**
     * Konstruktor mit Verbindungsobjekt und Versionstablle
     *
     * @param conn
     *            SQL Verbindungsobjekt
     * @param version
     *            Versionstabelle
     */
    protected OrVerwUpdateProvider(final Connection conn, final Version version) {

        this.conn = conn;
        this.version = version;
        final AbstractPlugin plugin = Application.getPluginLoader().getPlugin(OrdnerVerwaltung.class);
        this.manifest = plugin.getManifest();
        this.res = plugin.getResources();
    }

    /**
     * @see de.willuhn.sql.version.UpdateProvider#getConnection()
     */
    @Override
    public Connection getConnection() throws ApplicationException {

        return this.conn;
    }

    /**
     * @see de.willuhn.sql.version.UpdateProvider#getCurrentVersion()
     */
    @Override
    public int getCurrentVersion() throws ApplicationException {

        try {
            return this.version.getVersion();
        } catch (final RemoteException re) {
            Logger.error(I18N.tr(OrVerwUpdateProvider.TRANS_ERROR_READ_CURRENT_VERSION));
            throw new ApplicationException(I18N.tr(OrVerwUpdateProvider.TRANS_ERROR_READ_CURRENT_VERSION), re);
        }
    }

    /**
     * @see de.willuhn.sql.version.UpdateProvider#getProgressMonitor()
     */
    @Override
    public ProgressMonitor getProgressMonitor() {

        // Liefert den Splashscreen oder im Servermode einen
        // Pseudo-Monitor.
        return Application.getController().getApplicationCallback().getStartupMonitor();
    }

    /**
     * @return die PLuginResource
     */
    public PluginResources getResources() {

        return this.res;
    }

    /**
     * @see de.willuhn.sql.version.UpdateProvider#getUpdatePath()
     */
    @Override
    public File getUpdatePath() throws ApplicationException {

        // Ist das Unterverzeichnis "plugins" im Plugin
        return new File(this.manifest.getPluginDir(), "updates"); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.sql.version.UpdateProvider#setNewVersion(int)
     */
    @Override
    public void setNewVersion(final int newVersion) throws ApplicationException {

        final int current = this.getCurrentVersion();
        try {
            if (Logger.isLogging(Level.INFO)) {
                Logger.info(I18N.tr("OrVerwUpdateProvider.applyNewVersion", this.version.getName(), newVersion)); //$NON-NLS-1$
            }
            this.version.setVersion(newVersion);
            this.version.store();
        } catch (final RemoteException e) {
            // Im Fehlerfall Versionsnummer zuruecksetzen
            try {
                this.version.setVersion(current);
            } catch (final RemoteException e2) {
                Logger.error(I18N.tr("OrVerwUpdateProvider.unableToRollback"), e2); //$NON-NLS-1$
                // Werfen wir nicht, weil es sonst die eigentliche Exception
                // verdecken wuerde
            }
            Logger.error(I18N.tr(OrVerwUpdateProvider.TRANS_ERROR_READ_CURRENT_VERSION), e);
            throw new ApplicationException(I18N.tr(OrVerwUpdateProvider.TRANS_ERROR_READ_CURRENT_VERSION), e);
        } catch (final ApplicationException e) {
            // Im Fehlerfall Versionsnummer zuruecksetzen
            try {
                this.version.setVersion(current);
            } catch (final RemoteException e2) {
                Logger.error(I18N.tr("OrVerwUpdateProvider.unableToRollback"), e2); //$NON-NLS-1$
                // Werfen wir nicht, weil es sonst die eigentliche Exception
                // verdecken wuerde
            }
            throw e;
        }
    }
}
