/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.db.server;

import java.rmi.RemoteException;
import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;

import de.frankmuenster.jameica.orverw.Settings;
import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.gui.input.TextStyle;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.willuhn.datasource.db.AbstractDBObject;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.datasource.rmi.DBService;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementierung f&uuml; die Ordnertabelle
 *
 * <pre>
 * CREATE TABLE ordner (
 *   id NUMERIC default UNIQUEKEY('ordner'),
 *   title varchar(80) NOT NULL,
 *   titleFont varchar(80),
 *   titleColor int,
 *   titleAlignmnet int,
 *   content varchar(300),
 *   contentFont varchar(80),
 *   contentColor int,
 *   contentAlignmnet int,
 *   footer varchar(100),
 *   footerFont varchar(80),
 *   footerColor int,
 *   footerAlignmnet int,
 *   labelBackground int,
 *   horizontalBarVisible boolean default true,
 *   horizontalBarColor int,
 *   idVisible boolean default true,
 *   registerIndexFont varchar(80),
 *   registerIndexColor int,
 *   registerIndexBackground int,
 *   registerContentFont varchar(80),
 *   registerContentColor int,
 *   registerContentBackground int,
 *   width int,
 *   lastSaveDate date,
 *   UNIQUE (id),
 *   PRIMARY KEY (id)
 * );
 * </pre>
 *
 * @author Frank Mnster
 */
public class OrdnerImpl extends AbstractDBObject implements Ordner {

    /**
     * The serial UID
     */
    private static final long serialVersionUID = 1896984444569382973L;

    /**
     * Die korrete Anzahl an Font Attributen, die vorhanden sein m&uuml;ssen
     */
    private static final int CORRECT_FONT_ATTRIBUTE_LENNGTH = 4; // NOPMD

    /**
     * Standart Konstruktor
     *
     * @throws RemoteException
     */
    public OrdnerImpl() throws RemoteException {

        super();
    }

    /**
     * @see Ordner#getBarColor()
     */
    @Override
    public Color getBarColor() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("horizontalBarColor"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    /**
     * @see Ordner#getContent()
     */
    @Override
    public String getContent() throws RemoteException {

        return (String) this.getAttribute("content"); //$NON-NLS-1$
    }

    /**
     * @see de.frankmuenster.jameica.orverw.db.rmi.Ordner#getContentAlignment()
     */
    @Override
    public int getContentAlignment() throws RemoteException {

        return (Integer) this.getAttribute("contentAlignmnet"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#getContentColor()
     */
    @Override
    public Color getContentColor() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("contentColor"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    @Override
    public int getContentDirection() throws RemoteException {

        final Integer direction = (Integer) this.getAttribute("contentDirection"); //$NON-NLS-1$
        return direction == null ? SWT.HORIZONTAL : direction;
    }

    /**
     * @see Ordner#getContentFont()
     */
    @Override
    public Font getContentFont() throws RemoteException {

        final String fontAsString = (String) this.getAttribute("contentFont"); //$NON-NLS-1$
        return this.convertStringToFont(fontAsString);
    }

    /**
     * @see Ordner#getFooter()
     */
    @Override
    public String getFooter() throws RemoteException {

        return (String) this.getAttribute("footer"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#getFooterAlignment()
     */
    @Override
    public int getFooterAlignment() throws RemoteException {

        return (Integer) this.getAttribute("footerAlignmnet"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#getFooterColor()
     */
    @Override
    public Color getFooterColor() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("footerColor"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    @Override
    public int getFooterDirection() throws RemoteException {

        final Integer direction = (Integer) this.getAttribute("footerDirection"); //$NON-NLS-1$
        return direction == null ? SWT.HORIZONTAL : direction;
    }

    /**
     * @see Ordner#getFooterFont()
     */
    @Override
    public Font getFooterFont() throws RemoteException {

        final String fontAsString = (String) this.getAttribute("footerFont"); //$NON-NLS-1$
        return this.convertStringToFont(fontAsString);
    }

    /**
     * @see Ordner#getInhaltsverzeichniss()
     */
    @Override
    public DBIterator<Register> getInhaltsverzeichniss() throws RemoteException {

        try {
            // 1) Get the Database Service.
            final DBService service = this.getService();
            // 3) We create the history value list using getList(Class)
            final DBIterator<Register> register = service.createList(Register.class);
            // 4) we add a filter to only query for tasks with our project id
            register.addFilter("ordner_id = " + this.getID()); //$NON-NLS-1$
            register.setOrder("ORDER BY position asc"); //$NON-NLS-1$
            return register;
        } catch (final RemoteException e) {
            throw new RemoteException(I18N.tr("OrdnerImpl.ErrorLoadFolterContent"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see Ordner#getLabelBackground()
     */
    @Override
    public Color getLabelBackground() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("labelBackground"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    /**
     * @see Ordner#getLastSaveDate()
     */
    @Override
    public Date getLastSaveDate() throws RemoteException {

        return (Date) this.getAttribute("lastSaveDate"); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {

        return "id"; //$NON-NLS-1$
    }

    /**
     * @see Ordner#getRegisterContentBackground()
     */
    @Override
    public Color getRegisterContentBackground() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("registerContentBackground"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    /**
     * @see Ordner#getRegisterContentColor()
     */
    @Override
    public Color getRegisterContentColor() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("registerContentColor"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    /**
     * @see Ordner#getRegisterContentFont()
     */
    @Override
    public Font getRegisterContentFont() throws RemoteException {

        final String fontAsString = (String) this.getAttribute("registerContentFont"); //$NON-NLS-1$
        return this.convertStringToFont(fontAsString);
    }

    /**
     * @see Ordner#getRegisterIndexBackground()
     */
    @Override
    public Color getRegisterIndexBackground() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("registerIndexBackground"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    /**
     * @see Ordner#getRegisterIndexColor()
     */
    @Override
    public Color getRegisterIndexColor() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("registerIndexColor"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    /**
     * @see Ordner#getRegisterIndexFont()
     */
    @Override
    public Font getRegisterIndexFont() throws RemoteException {

        final String fontAsString = (String) this.getAttribute("registerIndexFont"); //$NON-NLS-1$
        return this.convertStringToFont(fontAsString);
    }

    /**
     * @see Ordner#getTitleAlignment()
     */
    @Override
    public int getTitleAlignment() throws RemoteException {

        return (Integer) this.getAttribute("titleAlignmnet"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#getTitleColor()
     */
    @Override
    public Color getTitleColor() throws RemoteException {

        final Integer rgb = (Integer) this.getAttribute("titleColor"); //$NON-NLS-1$
        return this.convertRgbIntToColor(rgb);
    }

    @Override
    public int getTitleDirection() throws RemoteException {
        final Integer direction = (Integer) this.getAttribute("titleDirection"); //$NON-NLS-1$
        return direction == null ? SWT.HORIZONTAL : direction;
    }

    /**
     * @see Ordner#getTitleFont()
     */
    @Override
    public Font getTitleFont() throws RemoteException {

        final String fontAsString = (String) this.getAttribute("titleFont"); //$NON-NLS-1$
        return this.convertStringToFont(fontAsString);
    }

    /**
     * @see Ordner#getTitleText()
     */
    @Override
    public String getTitleText() throws RemoteException {

        return (String) this.getAttribute("title"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#getWidth()
     */
    @Override
    public Integer getWidth() throws RemoteException {

        return (Integer) this.getAttribute("width"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#isBarVisible()
     */
    @Override
    public boolean isBarVisible() throws RemoteException {

        return (Boolean) this.getAttribute("horizontalBarVisible"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#isIdVisible()
     */
    @Override
    public boolean isIdVisible() throws RemoteException {

        return (Boolean) this.getAttribute("idVisible"); //$NON-NLS-1$
    }

    /**
     * @see Ordner#loadDefaults()
     */
    @Override
    public void loadDefaults() throws RemoteException {

        this.setBarColor(Settings.getDefaultBarColor());
        this.setBarVisible(Settings.isDefaultBarVisible());
        TextStyle style = Settings.getDefaultContentStyle();
        this.setContentAlignment(style.getAlignment());
        this.setContentColor(style.getTextColor());
        this.setContentFont(style.getFont());
        style = Settings.getDefaultFooterStyle();
        this.setFooterAlignment(style.getAlignment());
        this.setFooterColor(style.getTextColor());
        this.setFooterFont(style.getFont());
        this.setIdVisible(Settings.isDefaultIdVisible());
        this.setLabelBackground(Settings.getDefaultLabelBackground());
        style = Settings.getDefaultRegisterContentStyle();
        this.setRegisterContentBackground(style.getBackgroundColor());
        this.setRegisterContentColor(style.getTextColor());
        this.setRegisterContentFont(style.getFont());
        style = Settings.getDefaultRegisterIndexStyle();
        this.setRegisterIndexBackground(style.getBackgroundColor());
        this.setRegisterIndexColor(style.getTextColor());
        this.setRegisterIndexFont(style.getFont());
        style = Settings.getDefaultTitleStyle();
        this.setTitleAlignment(style.getAlignment());
        this.setTitleColor(style.getTextColor());
        this.setTitleFont(style.getFont());
        // setzte ein paar Dummy Texte
        this.setContent(I18N.tr("OrdnerImpl.defaultContent")); //$NON-NLS-1$
        this.setFooter(I18N.tr("OrdnerImpl.defaultFooter")); //$NON-NLS-1$
        this.setTitleText(I18N.tr("OrdnerImpl.defaultTitel")); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setBarColor(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setBarColor(final Color color) throws RemoteException {

        this.setAttribute("horizontalBarColor", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setBarVisible(boolean)
     */
    @Override
    public void setBarVisible(final boolean visible) throws RemoteException {

        this.setAttribute("horizontalBarVisible", visible); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setContent(java.lang.String)
     */
    @Override
    public void setContent(final String inhalt) throws RemoteException {

        this.setAttribute("content", inhalt); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setContentAlignment(int)
     */
    @Override
    public void setContentAlignment(final int alignment) throws RemoteException {

        this.setAttribute("contentAlignmnet", alignment); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setContentColor(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setContentColor(final Color color) throws RemoteException {

        this.setAttribute("contentColor", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    @Override
    public void setContentDirection(final int direction) throws RemoteException {
        this.setAttribute("contentDirection", direction); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setContentFont(org.eclipse.swt.graphics.Font)
     */
    @Override
    public void setContentFont(final Font font) throws RemoteException {

        this.setAttribute("contentFont", this.convertFontToString(font)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setFooter(java.lang.String)
     */
    @Override
    public void setFooter(final String fussnote) throws RemoteException {

        this.setAttribute("footer", fussnote); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setFooterAlignment(int)
     */
    @Override
    public void setFooterAlignment(final int alignment) throws RemoteException {

        this.setAttribute("footerAlignmnet", alignment); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setFooterColor(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setFooterColor(final Color color) throws RemoteException {

        this.setAttribute("footerColor", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    @Override
    public void setFooterDirection(final int direction) throws RemoteException {
        this.setAttribute("footerDirection", direction); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setFooterFont(org.eclipse.swt.graphics.Font)
     */
    @Override
    public void setFooterFont(final Font font) throws RemoteException {

        this.setAttribute("footerFont", this.convertFontToString(font)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setIdVisible(boolean)
     */
    @Override
    public void setIdVisible(final boolean visible) throws RemoteException {

        this.setAttribute("idVisible", visible); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setLabelBackground(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setLabelBackground(final Color color) throws RemoteException {

        this.setAttribute("labelBackground", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setLastSaveDate(java.util.Date)
     */
    @Override
    public void setLastSaveDate(final Date datum) throws RemoteException {

        this.setAttribute("lastSaveDate", datum); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setRegisterContentBackground(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setRegisterContentBackground(final Color color) throws RemoteException {

        this.setAttribute("registerContentBackground", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setRegisterContentColor(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setRegisterContentColor(final Color color) throws RemoteException {

        this.setAttribute("registerContentColor", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setRegisterContentFont(org.eclipse.swt.graphics.Font)
     */
    @Override
    public void setRegisterContentFont(final Font font) throws RemoteException {

        this.setAttribute("registerContentFont", this.convertFontToString(font)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setRegisterIndexBackground(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setRegisterIndexBackground(final Color color) throws RemoteException {

        this.setAttribute("registerIndexBackground", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setRegisterIndexColor(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setRegisterIndexColor(final Color color) throws RemoteException {

        this.setAttribute("registerIndexColor", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setRegisterIndexFont(org.eclipse.swt.graphics.Font)
     */
    @Override
    public void setRegisterIndexFont(final Font font) throws RemoteException {

        this.setAttribute("registerIndexFont", this.convertFontToString(font)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setTitleAlignment(int)
     */
    @Override
    public void setTitleAlignment(final int alignment) throws RemoteException {

        this.setAttribute("titleAlignmnet", alignment); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setTitleColor(org.eclipse.swt.graphics.Color)
     */
    @Override
    public void setTitleColor(final Color color) throws RemoteException {

        this.setAttribute("titleColor", this.convertColorToRgbInt(color)); //$NON-NLS-1$
    }

    @Override
    public void setTitleDirection(final int direction) throws RemoteException {
        this.setAttribute("titleDirection", direction); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setTitleFont(org.eclipse.swt.graphics.Font)
     */
    @Override
    public void setTitleFont(final Font font) throws RemoteException {

        this.setAttribute("titleFont", this.convertFontToString(font)); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setTitleText(java.lang.String)
     */
    @Override
    public void setTitleText(final String titel) throws RemoteException {

        this.setAttribute("title", titel); //$NON-NLS-1$
    }

    /**
     * @see Ordner#setWidth(java.lang.Integer)
     */
    @Override
    public void setWidth(final Integer width) throws RemoteException {

        this.setAttribute("width", width); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#deleteCheck()
     */
    @Override
    protected void deleteCheck() throws ApplicationException {

        // nothing to do here
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#getTableName()
     */
    @Override
    protected String getTableName() {

        return "ordner"; //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#insertCheck()
     */
    @Override
    protected void insertCheck() throws ApplicationException {

        try {
            if (this.getTitleText() == null) {
                this.setTitleText(StringUtils.EMPTY);
            }
            if (this.getContent() == null) {
                this.setContent(StringUtils.EMPTY);
            }
            if (this.getFooter() == null) {
                this.setFooter(StringUtils.EMPTY);
            }
            if (this.getLastSaveDate() == null) {
                this.setLastSaveDate(new Date());
            }
        } catch (final RemoteException e) {
            Logger.error("insert check of ordner failed", e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("OrdnerImpl.errorStoreFolder"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see de.willuhn.datasource.db.AbstractDBObject#updateCheck()
     */
    @Override
    protected void updateCheck() throws ApplicationException {

        // nearby same as insertCheck
        this.insertCheck();

        // An update lastSaveDate to today
        try {
            this.setLastSaveDate(new Date());
        } catch (final RemoteException e) {
            Logger.error("upadte of lastSaveDate failed", e); //$NON-NLS-1$
            throw new ApplicationException(I18N.tr("OrdnerImpl.errorStoreFolder"), e); //$NON-NLS-1$
        }
    }

    private int convertColorToRgbInt(final Color color) {

        int rgb;
        if (color == null) {
            rgb = -1;
        } else {
            // create long aus RGB
            rgb = 65536 * color.getRed() + 256 * color.getGreen() + color.getBlue();
        }
        return rgb;
    }

    private String convertFontToString(final Font font) {

        String converted;
        if (font == null) {
            converted = StringUtils.EMPTY;
        } else {
            final StringBuilder fontData = new StringBuilder(30);
            final FontData fontdata = font.getFontData()[0];
            fontData.append(fontdata.getHeight()).append(';').append(fontdata.getLocale()).append(';')
                    .append(fontdata.getName()).append(';').append(fontdata.getStyle()).append(';');
            converted = fontData.toString();
        }
        return converted;
    }

    private Color convertRgbIntToColor(final Integer rgbInt) {

        Color color;
        if (rgbInt == null) {
            color = null; // NOPMD
        } else {
            final int red = rgbInt % 0x1000000 / 0x10000;
            final int green = rgbInt % 0x10000 / 0x100;
            final int blue = rgbInt % 0x100;
            color = new Color(GUI.getDisplay(), red, green, blue);
        }
        return color;
    }

    private Font convertStringToFont(final String fontAsString) {

        Font font;
        if (fontAsString == null) {
            font = GUI.getShell().getFont();
        } else {
            final String[] fontAttributtes = fontAsString.split(";"); //$NON-NLS-1$
            if (fontAttributtes.length == OrdnerImpl.CORRECT_FONT_ATTRIBUTE_LENNGTH) {
                final FontData fontdata = new FontData();
                fontdata.setHeight(Integer.valueOf(fontAttributtes[0]));
                fontdata.setLocale(fontAttributtes[1]);
                fontdata.setName(fontAttributtes[2]);
                fontdata.setStyle(Integer.valueOf(fontAttributtes[3]));
                font = new Font(GUI.getDisplay(), fontdata);
            } else {
                font = GUI.getShell().getFont();
            }
        }
        return font;
    }
}
