/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.parts.labelpart;

import java.rmi.RemoteException;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import de.frankmuenster.jameica.orverw.gui.input.LabelInputEnhanced;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelButtonListener;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelDirectionButtons;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelGeneralButtons;
import de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum;
import de.frankmuenster.jameica.orverw.parts.labelpart.trenner.BarTrenner;
import de.frankmuenster.jameica.orverw.util.Utils;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.gui.util.SimpleContainer;

/**
 * Das Part f&uuml;r den Ordnerr&uuml;cken.
 *
 * @author Frank Mnster
 */
public final class LabelPart implements Part, LabelButtonListener, LabelInputTextChangeListener {

    /**
     * Left and right margin of text section in millimetre
     */
    public static final int TEXT_LEFT_RIGHT_MARGIN = 4;

    /**
     * Die H&ouml;he der oberen Linie
     */
    public static final int TB_HEIGHT = 40;

    /**
     * Das obere Margin der oberen Linie
     */
    public static final int TB_TOP_MARGIN = 30;

    /**
     * Die H&ouml;he der mittleren Linie
     */
    public static final int MB_HEIGHT = 20;

    /**
     * Das obere Margin der mittleren Linie
     */
    public static final int MB_TOP_MARGIN = 10;

    /**
     * Die Linienh&ouml;e
     */
    public static final int BAR_HEIGHT = 6;

    /**
     * Die default Titeltextausrichtung
     */
    private static final int TITEL_ALIGNMENT = SWT.CENTER;

    /**
     * Der Hauptcontainer f&uuml;r den Ordnerr&uuml;cken
     */
    private transient SimpleContainer label;

    /**
     * background colour
     */
    private Color bgColour;

    /**
     * Die obere Linie
     */
    private final transient BarTrenner bar1;

    /**
     * Die untere Linie
     */
    private final transient BarTrenner bar2;

    /**
     * Sichtbarkeit der Linien
     */
    private boolean barVisible;

    /**
     * Farbe der Linien
     */
    private Color barColour;

    /**
     * Eingabefeld f&uuml;r den Titeltext
     */
    private final transient LabelInputText titleTextPart;

    /**
     * Der Text des Titelfeldes
     */
    private String titelText;

    /**
     * Eingabefeld f&uuml;r den Hauptbereiches
     */
    private final transient LabelInputText contentTextPart;

    /**
     * Der Text des Hauptbereiches
     */
    private String contentText;

    /**
     * Eingabefeld f&uuml;r den Fu&szlig;nereich
     */
    private final transient LabelInputText footerTextPart;

    /**
     * Des des Fu&szlig;nereiches
     */
    private String footerText;

    /**
     * Label f&uuml;r die ID
     */
    private final LabelInputEnhanced idCLabel;

    /**
     * Der Text der ID
     */
    private String idText;

    /**
     * Sichtbarkeit der ID.
     */
    private boolean idVisible;

    /**
     * The label size
     */
    private LabelSizeEnum labelSize;

    /**
     * Die Listener fr einen Button-Click-Event
     */
    private final transient Set<LabelPartListener> listeners = new CopyOnWriteArraySet<>();

    /**
     * Erstellt eine neue Ordnerbeschriftung mit der angegeben Breite.
     *
     * @param labelSize
     *            - Breite des Labels in Millimeter
     * @param listener
     *            - Listener fr Knpfe
     */
    public LabelPart(final LabelSizeEnum labelSize, final LabelPartListener listener) {

        if (labelSize == null) {
            this.labelSize = LabelSizeEnum.getDefault();
        } else {
            this.labelSize = labelSize;
        }
        this.barVisible = true;
        this.idVisible = true;

        this.addListener(listener);

        // calculate sizes
        final Point titleSize = getSectionSizeInPixel(LabelPartSectionsEnum.TITLE, this.labelSize);
        final Point contentSize = getSectionSizeInPixel(LabelPartSectionsEnum.CONTENT, this.labelSize);
        final Point footerSize = getSectionSizeInPixel(LabelPartSectionsEnum.FOOTER, this.labelSize);

        final Point topBarSize = new Point(titleSize.x, LabelPart.TB_HEIGHT + LabelPart.BAR_HEIGHT);
        this.bar1 = new BarTrenner(topBarSize, LabelPart.TB_TOP_MARGIN, LabelPart.BAR_HEIGHT, this.getBarColour(),
                this.getBgColour());

        // title section
        this.titleTextPart = new LabelInputText(titleSize, LabelPart.TITEL_ALIGNMENT, SWT.RIGHT);
        this.titleTextPart.setCenterVertical(true);
        this.titleTextPart.addListener(this);

        final Point middleBarSize = new Point(titleSize.x, LabelPart.MB_HEIGHT + LabelPart.BAR_HEIGHT);
        this.bar2 = new BarTrenner(middleBarSize, LabelPart.MB_TOP_MARGIN, LabelPart.BAR_HEIGHT, this.getBarColour(),
                this.getBgColour());
        this.contentTextPart = new LabelInputText(contentSize, SWT.LEFT, SWT.LEFT);
        this.contentTextPart.addListener(this);
        this.footerTextPart = new LabelInputText(footerSize);
        this.footerTextPart.addListener(this);

        // create the ID label
        this.idCLabel = new LabelInputEnhanced(StringUtils.EMPTY, titleSize.x);
        this.idCLabel.setTextAlignment(SWT.RIGHT);

    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            der hinzugefgt werden soll
     */
    public void addListener(final LabelPartListener listener) {

        this.listeners.add(listener);
    }

    /**
     * Liefert die Textausrichtung des angegebenen Eingabefeldes.
     *
     * @param section
     *            das Eingabefeld
     * @return Die Textausrichtung
     */
    public int getAlignment(final LabelPartSectionsEnum section) {

        int alignment;

        if (section == null) {
            alignment = SWT.LEFT;
        } else {
            switch (section) {
            case CONTENT:
                alignment = this.contentTextPart.getTextAlignment();
                break;
            case FOOTER:
                alignment = this.footerTextPart.getTextAlignment();
                break;
            case TITLE:
                alignment = this.titleTextPart.getTextAlignment();
                break;
            default:
                alignment = SWT.LEFT;
                break;
            }
        }
        return alignment;
    }

    /**
     * Liefert die Farbe der Linien. Wurde noch keine Farbe gesetzt wird sie auf
     * RGB(0, 0, 240) gesetzt.
     *
     * @return Die Farbe der Linien
     */
    public Color getBarColour() {

        if (this.barColour != null) {
            this.barColour = new Color(GUI.getDisplay(), 0, 0, 240);
        }
        return this.barColour;
    }

    /**
     * Liefert die Sichtbarkeit der Linien
     *
     * @return Die Sichtbarkeit der Linien
     */
    public Boolean getBarVisible() {

        return this.barVisible;
    }

    /**
     * Liefert die Hintergrundfarbe des Labels. Wurde noch keine gesetzt wird
     * sie auf RGB(255, 255, 224) gesetzt.
     *
     * @return Die Hintergrundfarbe des Labels
     */
    public Color getBgColour() {

        if (this.bgColour == null) {
            this.bgColour = new Color(GUI.getDisplay(), 255, 255, 224);
        }
        return this.bgColour;
    }

    /**
     * Liefert den Text des Hauptfeldes
     *
     * @return Den Text des Hauptfeldes
     */
    public String getContentText() {

        return this.contentTextPart.getText();
    }

    /**
     * Returns the direction of the requested area
     *
     * @param section
     *            das Eingabefeld
     * @return Die Textausrichtung
     */
    public int getDirection(final LabelPartSectionsEnum section) {

        int direction;

        if (section == null) {
            direction = SWT.HORIZONTAL;
        } else {
            switch (section) {
            case TITLE:
                direction = this.titleTextPart.getTextDirection();
                break;
            case CONTENT:
                direction = this.contentTextPart.getTextDirection();
                break;
            case FOOTER:
                direction = this.footerTextPart.getTextDirection();
                break;
            default:
                direction = SWT.HORIZONTAL;
                break;
            }
        }
        return direction;
    }

    /**
     * Liefert die Textfarbe in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @return Die Textfarbe
     */
    public Color getFgColour(final LabelPartSectionsEnum section) {

        Color colour;

        if (section == null) {
            colour = GUI.getShell().getForeground();
        } else {
            switch (section) {
            case TITLE:
                colour = this.titleTextPart.getForeground();
                break;
            case CONTENT:
                colour = this.contentTextPart.getForeground();
                break;
            case FOOTER:
                colour = this.footerTextPart.getForeground();
                break;
            default:
                colour = GUI.getShell().getForeground();
                break;
            }
        }

        return colour;
    }

    /**
     * Liefert den Schrifttyp ein dem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @return Den Schrifttyp
     */
    public Font getFont(final LabelPartSectionsEnum section) {

        Font font;

        if (section == null) {
            font = GUI.getShell().getFont();
        } else {
            switch (section) {
            case CONTENT:
                font = this.contentTextPart.getFont();
                break;
            case FOOTER:
                font = this.footerTextPart.getFont();
                break;
            case TITLE:
                font = this.titleTextPart.getFont();
                break;
            default:
                font = GUI.getShell().getFont();
                break;
            }
        }
        return font;
    }

    /**
     * Liefert den Text des Fu&szlig;feldes
     *
     * @return Den Text des Fu&szlig;feldes
     */
    public String getFooterText() {

        return this.footerTextPart.getText();
    }

    /**
     * Liefert die ID
     *
     * @return Die ID
     */
    public String getIdText() {

        return this.idText;
    }

    /**
     * Liefert die Breite des Labels
     *
     * @return die Breite des Labels
     */
    public LabelSizeEnum getLabelSize() {

        return this.labelSize;
    }

    /**
     * Liefert den Text des Titelfeldes
     *
     * @return Den Text des Titelfeldes
     */
    public String getTitelText() {

        return this.titleTextPart.getText();
    }

    /**
     * Liefert die Sichtbarkeit der ID
     *
     * @return Die Sichtbarkeit der ID
     */
    public boolean isIdVisible() {

        return this.idVisible;
    }

    /**
     * @see de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelButtonListener#notifyInputButtonClicked(de.frankmuenster.jameica.orverw.parts.labelpart.LabelPartSectionsEnum,
     *      de.frankmuenster.jameica.orverw.parts.labelpart.buttons.LabelPartButtonsEnum)
     */
    @Override
    public void notifyInputButtonClicked(final LabelPartSectionsEnum section, final LabelPartButtonsEnum button) {

        if (section != null) {
            Font font;
            Color colour;

            switch (section) {
            case CONTENT:
                font = this.contentTextPart.getFont();
                colour = this.contentTextPart.getForeground();
                break;
            case FOOTER:
                font = this.footerTextPart.getFont();
                colour = this.footerTextPart.getForeground();
                break;
            case TITLE:
                font = this.titleTextPart.getFont();
                colour = this.titleTextPart.getForeground();
                break;
            case GENERAL_BUTTONS:
                font = null; // NOPMD
                switch (button) {
                case BAR_COLOUR:
                    colour = this.getBarColour();
                    break;
                case BACKGROUND_COLOUR:
                    colour = this.getBgColour();
                    break;
                default:
                    colour = null; // NOPMD
                    break;
                }
                break;
            default:
                font = new Font(GUI.getDisplay(), new FontData());
                colour = new Color(GUI.getDisplay(), new RGB(0, 0, 0));
                break;
            }
            this.notifyListenersButtonClicked(new LabelPartButtonObject(section, button, font, colour));

        }
    }

    /**
     * @see de.frankmuenster.jameica.orverw.parts.labelpart.LabelInputTextChangeListener#notifyTextChanged()
     */
    @Override
    public void notifyTextChanged() {

        this.notifyListenersTextChanged();
    }

    /**
     * @see de.willuhn.jameica.gui.Part#paint(org.eclipse.swt.widgets.Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        this.label = new SimpleContainer(parent, false);
        this.paintPart();

    }

    /**
     * Entfernd einen Listener aus der Liste
     *
     * @param listener
     *            der entfernt werden soll
     */
    public void removeListener(final LabelPartListener listener) {

        this.listeners.remove(listener);
    }

    /**
     * Setzt die Textausrichtung in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld.
     * @param alignment
     *            die Textausrichtung
     */
    public void setAlignment(final LabelPartSectionsEnum section, final int alignment) {

        if (section != null) {
            switch (section) {
            case CONTENT:
                this.contentTextPart.setTextAlignment(alignment);
                break;
            case FOOTER:
                this.footerTextPart.setTextAlignment(alignment);
                break;
            case TITLE:
                this.titleTextPart.setTextAlignment(alignment);
                break;
            default:
                break;
            }
        }
    }

    /**
     * Setzt die Farbe der Linien
     *
     * @param colour
     *            die Farbe der Linien
     */
    public void setBarColour(final Color colour) {

        if (colour != null) {
            this.barColour = colour;
            if (this.bar1 != null) {
                this.bar1.setBarColor(colour);
            }
            if (this.bar2 != null) {
                this.bar2.setBarColor(colour);
            }
        }
    }

    /**
     * Setzt die Sichtbarkeit der Linien
     *
     * @param lineVisible
     *            die Sichtbarkeit der Linien
     */
    public void setBarVisible(final boolean lineVisible) {

        this.barVisible = lineVisible;
        if (this.bar1 != null) {
            this.bar1.setVisible(this.barVisible);
        }
        if (this.bar2 != null) {
            this.bar2.setVisible(this.barVisible);
        }
    }

    /**
     * Setzt die Hintergrundfarbe des Labels.
     *
     * @param colour
     *            die Hintergrundfarbe des Labels.
     */
    public void setBgColour(final Color colour) {

        if (colour != null) {
            this.bgColour = colour;
            if (this.label != null) {
                this.setBackgrounds();
            }
        }
    }

    /**
     * Setzt den Text des Hauptfeldes
     *
     * @param content
     *            der Text des Hauptfeldes
     */
    public void setContentText(final String content) {

        if (content != null) {
            this.contentText = content;
            if (this.contentTextPart != null) {
                this.contentTextPart.setText(content);
            }
        }
    }

    /**
     * Set the text direction for the given section. Can be SWT.LEFT, SWT.RIGHT
     * or HORIZONTAL. All other values will be set to SWT.HORIZONTAL.
     *
     * @param section
     *            the {@link LabelPartSectionsEnum}
     * @param direction
     *            the direction
     */
    public void setDirection(final LabelPartSectionsEnum section, final int direction) {

        if (section != null) {
            // now set direction
            switch (section) {
            case CONTENT:
                this.contentTextPart.setTextDirection(direction);
                break;
            case FOOTER:
                this.footerTextPart.setTextDirection(direction);
                break;
            case TITLE:
                this.titleTextPart.setTextDirection(direction);
                break;
            default:
                break;
            }
        }
    }

    /**
     * Setzt den Schrifttyp in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @param font
     *            den Schrifttyp
     */
    public void setFont(final LabelPartSectionsEnum section, final Font font) {

        if (section != null) {
            switch (section) {
            case CONTENT:
                this.contentTextPart.setFont(font);
                break;
            case FOOTER:
                this.footerTextPart.setFont(font);
                break;
            case TITLE:
                this.titleTextPart.setFont(font);
                break;
            default:
                break;
            }
        }
    }

    /**
     * Setzt den Text des Fu&szlig;feldes
     *
     * @param footer
     *            Der Text des Fu&szlig;feldes
     */
    public void setFooterText(final String footer) {

        if (footer != null) {
            this.footerText = footer;
            if (this.footerTextPart != null) {
                this.footerTextPart.setText(footer);
            }
        }
    }

    /**
     * Setzt die Textfarbe in einem angegeben Eingabefeld.
     *
     * @param section
     *            das Eingabefeld
     * @param color
     *            die Farbe
     */
    public void setForegroundColor(final LabelPartSectionsEnum section, final Color color) {

        if (section != null) {
            switch (section) {
            case CONTENT:
                this.contentTextPart.setForeground(color);
                break;
            case FOOTER:
                this.footerTextPart.setForeground(color);
                break;
            case TITLE:
                this.titleTextPart.setForeground(color);
                break;
            default:
                break;
            }
        }

    }

    /**
     * Setzt die ID
     *
     * @param idText
     *            Die ID
     */
    public void setIdText(final String idText) {

        this.idText = idText;
        if (this.idCLabel != null) {
            this.fillText();
        }
    }

    /**
     * Setzt die Sichtbarkeit der ID
     *
     * @param visible
     *            die Sichtbarkeit der ID
     */
    public void setIdVisible(final boolean visible) {

        this.idVisible = visible;
        if (this.idCLabel != null) {
            this.idCLabel.setVisible(visible);
        }
    }

    /**
     * @param titelAlignment
     */
    public void setTitleAlignment(final int titelAlignment) {

        this.titleTextPart.setTextAlignment(titelAlignment);
    }

    /**
     * Setzt den Text des Titelfeldes
     *
     * @param titel
     *            der Text des Titelfeldes
     */
    public void setTitleText(final String titel) {

        if (titel != null) {
            this.titelText = titel;
            if (this.titleTextPart != null) {
                this.titleTextPart.setText(titel);
            }
        }
    }

    /**
     * Aktualisiert die Texte in den Eingabefeldern
     */
    private void fillText() {

        if (this.titleTextPart != null) {
            this.titleTextPart.setText(this.titelText);
        }
        if (this.contentTextPart != null) {
            this.contentTextPart.setText(this.contentText);
        }
        if (this.footerTextPart != null) {
            this.footerTextPart.setText(this.footerText);
        }
        if (this.idCLabel != null) {
            this.idCLabel.setValue("ID " + this.idText); //$NON-NLS-1$
        }

    }

    /**
     * Calculates the width and the height of a section in pixel on the display
     * and returns a {@link Point} with x ad height and y as width.
     *
     * @param section
     *            the section of type {@link LabelPartSectionsEnum}
     * @param labelWidth
     *            the label width in millimetre
     *
     * @return a {@link Point} with the x as height and y as width in pixel
     */
    private Point getSectionSizeInPixel(final LabelPartSectionsEnum section, final LabelSizeEnum labelSize) {

        // Width depends on the label size
        final int width = Utils.mmToDisplayPoints(labelSize.getWidth());
        // height depends on the section
        final int height = Utils.mmToDisplayPoints(section.getHeight());
        return new Point(width, height);
    }

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe des angeklickten
     * Buttons
     *
     * @param button
     */
    private void notifyListenersButtonClicked(final LabelPartButtonObject button) {

        for (final LabelPartListener listener : this.listeners) {
            listener.notifyButtonClicked(button);
        }
    }

    /**
     * Benachrichtigen der registrierten Listeners das der Text ge&auml;ndert
     * wurde.
     */
    private void notifyListenersTextChanged() {

        for (final LabelPartListener listener : this.listeners) {
            listener.notifyLabelTextChanged();
        }
    }

    /**
     * Die eigentliche Zeichenroutine.
     */
    private void paintPart() {

        // Texte
        this.fillText();
        this.setBackgrounds();

        final GridLayout gridLayout = new GridLayout();
        gridLayout.marginWidth = 0;
        gridLayout.marginHeight = 0;
        gridLayout.verticalSpacing = 0;
        gridLayout.numColumns = 5;
        this.label.getComposite().setLayout(gridLayout);

        // first row,first column: empty (spans over all 6 rows)
        final Label emptyLeft = new Label(this.label.getComposite(), SWT.NONE);
        emptyLeft.setText(StringUtils.EMPTY);
        GridData gridData = new GridData();
        gridData.horizontalIndent = 0;
        gridData.verticalIndent = 0;
        gridData.verticalSpan = 6;
        gridData.verticalAlignment = GridData.FILL;
        gridData.widthHint = 30;
        emptyLeft.setLayoutData(gridData);

        // first row, second column: starts with the upper bar
        this.bar1.paint(this.label.getComposite());

        // first row, third column: empty
        new Label(this.label.getComposite(), SWT.BORDER);

        // [1-6:4] empty (spans over all 6 rows)
        final Label emptyRight = new Label(this.label.getComposite(), SWT.BORDER);
        emptyRight.setText(StringUtils.EMPTY);
        gridData = new GridData();
        gridData.horizontalIndent = 0;
        gridData.verticalIndent = 0;
        gridData.verticalSpan = 6;
        gridData.verticalAlignment = GridData.FILL;
        gridData.grabExcessHorizontalSpace = true;
        // gridData.widthHint = 30;
        emptyRight.setLayoutData(gridData);

        // first row, last cell: empty
        new Label(this.label.getComposite(), SWT.NONE);

        // second row second cell: title
        this.titleTextPart.paint(this.label.getComposite());

        // [2:3] font style selector for the title
        new LabelDirectionButtons(this.label.getComposite(), LabelPartSectionsEnum.TITLE, this);

        // [2-5:5] general buttons (spans over the last 4 rows)
        final LabelGeneralButtons generalButtons = new LabelGeneralButtons(this.label.getComposite(),
                LabelPartSectionsEnum.GENERAL_BUTTONS, this);
        gridData = new GridData();
        gridData.horizontalIndent = 0;
        gridData.verticalIndent = 0;
        gridData.verticalSpan = 4;
        gridData.verticalAlignment = GridData.FILL;
        generalButtons.setLayoutData(gridData);

        // [3:2] lower bar
        this.bar2.paint(this.label.getComposite());

        // [3:3] empty
        new Label(this.label.getComposite(), SWT.NONE);

        // [4:2] Context label
        this.contentTextPart.paint(this.label.getComposite());

        // [4:3] font style selector for the content
        new LabelDirectionButtons(this.label.getComposite(), LabelPartSectionsEnum.CONTENT, this);

        // [5:2] footer label
        this.footerTextPart.paint(this.label.getComposite());
        // [5:3] font style selector for the footer
        new LabelDirectionButtons(this.label.getComposite(), LabelPartSectionsEnum.FOOTER, this);

        // [6:2] id of the file
        this.idCLabel.paint(this.label.getComposite(), 10);

        this.label.getComposite().layout();
        this.label.getComposite().pack();

    }

    /**
     * Setzt die Hintergrundfarbe aller Widgets
     */
    private void setBackgrounds() {

        this.bar1.setBackground(this.getBgColour());
        this.bar2.setBackground(this.getBgColour());
        this.titleTextPart.setBackground(this.getBgColour());
        this.contentTextPart.setBackground(this.getBgColour());
        this.footerTextPart.setBackground(this.getBgColour());
        this.idCLabel.setBackground(this.getBgColour());
    }

}
