/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.orverw.print;

import java.rmi.RemoteException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.eclipse.jface.resource.FontDescriptor;
import org.eclipse.nebula.paperclips.core.EmptyPrint;
import org.eclipse.nebula.paperclips.core.ImagePrint;
import org.eclipse.nebula.paperclips.core.LayerPrint;
import org.eclipse.nebula.paperclips.core.Margins;
import org.eclipse.nebula.paperclips.core.PaperClips;
import org.eclipse.nebula.paperclips.core.Print;
import org.eclipse.nebula.paperclips.core.grid.DefaultGridLook;
import org.eclipse.nebula.paperclips.core.grid.GridPrint;
import org.eclipse.nebula.paperclips.core.page.PageNumber;
import org.eclipse.nebula.paperclips.core.page.PagePrint;
import org.eclipse.nebula.paperclips.core.text.TextPrint;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.printing.PrinterData;

import de.frankmuenster.jameica.orverw.db.rmi.Ordner;
import de.frankmuenster.jameica.orverw.db.rmi.Register;
import de.frankmuenster.jameica.orverw.util.I18N;
import de.frankmuenster.jameica.orverw.util.Utils;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Implementiert den Ausdruck des Registers.
 *
 * @author Frank Mnster
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.BeanMembersShouldSerialize" })
public class PrintRegister extends AbstractPrint {

    /**
     * Width of index field in px
     */
    private static final int INDEX_WIDTH = 40;

    /**
     * Maximal width of description field in px
     */
    public static final int DESCRIPTION_WIDTH = 340;

    /**
     * Defines a 72 points per inch point
     */
    private static final Point DPI_72 = new Point(72, 72);

    /**
     * Calculation factor screen DPI to 72 DPI
     */
    private static final double FACTOR = GUI.getDisplay().getDPI().x / (double) PrintRegister.DPI_72.x;

    /**
     * Der Registerinhalt
     */
    private final DBIterator<Register> index;

    /**
     * Der Ordner Titel
     */
    private final String title;

    /**
     * Die Ordner ID
     */
    private final String ordnerID;

    /**
     * Das letzte Speicherdatum
     */
    private final Date lastSaveDate;

    /**
     * Der Titeltext Schrifttyp
     */
    private final Font titleFont;

    /**
     * der Indextext Schrifttyp
     */
    private final Font indexFont;

    /**
     * Der Beschreibungstext Schrifttyp
     */
    private final Font contentFont;

    /**
     * Die Hintergrundfarbe des Indexfeldes
     */
    private final Color indexBgColor;

    /**
     * Die Hintergrundfarbe des Beschreibungsfeldes
     */
    private final Color contentBgColor;

    /**
     * Der Konstruktor
     *
     * @param ordner
     *            der Ordner dessen Register gedruckt werden soll.
     * @throws ApplicationException
     */
    public PrintRegister(final Ordner ordner) throws ApplicationException {

        super();
        try {
            this.index = ordner.getInhaltsverzeichniss();
            this.title = ordner.getTitleText();
            this.ordnerID = ordner.getID();
            this.lastSaveDate = ordner.getLastSaveDate();
            this.titleFont = ordner.getTitleFont();
            this.indexFont = ordner.getRegisterIndexFont();
            this.contentFont = ordner.getRegisterContentFont();
            this.indexBgColor = ordner.getRegisterIndexBackground();
            this.contentBgColor = ordner.getRegisterContentBackground();
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("PrintRegister.errorCreatingPrintObj"), e); //$NON-NLS-1$
        }
    }

    /**
     * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#getTitle()
     */
    @Override
    public String getTitle() {

        return Utils.format(this.title, 10);
    }

    /**
     * @see AbstractPrint#customise(PagePrint)
     */
    @Override
    protected void customise(final PagePrint page) throws ApplicationException {

        // Neuer Header mit ID
        page.setHeader((final PageNumber pageNumber) -> {

            final GridPrint header = new GridPrint("d:g,r:d"); //$NON-NLS-1$
            final TextPrint title1 = new TextPrint(StringUtils.EMPTY, AbstractPrint.FONT_TINY, SWT.LEFT);
            final TextPrint title2 = new TextPrint("ID " + this.ordnerID, AbstractPrint.FONT_TINY, //$NON-NLS-1$
                    SWT.RIGHT);
            final TextPrint title3 = new TextPrint(this.title, this.titleFont.getFontData()[0]);
            header.add(SWT.RIGHT, title2, 2);
            header.add(SWT.LEFT, title3, 2);
            header.add(SWT.RIGHT, title1, 2);
            header.add(SWT.RIGHT, title1, 2);

            return header;
        });

        // neuer Foot mit Datum und Seitenangeben
        page.setFooter((final PageNumber pageNumber) -> {
            final GridPrint footer = new GridPrint("d:g, r:d"); //$NON-NLS-1$
            final TextPrint datum = new TextPrint(this.getLastSaveDate(), AbstractPrint.FONT_TINY, SWT.LEFT);
            footer.add(datum);

            return footer;
        });

    }

    /**
     * @see de.frankmuenster.jameica.orverw.print.AbstractPrint#getPrintContent()
     */
    @Override
    @SuppressWarnings({ "PMD.ExcessiveMethodLength" })
    protected Print getPrintContent() throws ApplicationException {

        // 2 cm margin
        final Margins margin = new Margins(Utils.mmToPoints(20));
        margin.left = Utils.mmToPoints(40);

        final PrinterData data = Printer.getDefaultPrinterData();
        if (data == null) {
            throw new ApplicationException(I18N.tr("PrintRegister.noPrinterFound")); //$NON-NLS-1$
        }
        final Printer printer = new Printer(data);

        final DefaultGridLook look = new DefaultGridLook();
        look.setCellSpacing(5, 2);
        final GridPrint grid = new GridPrint("50, r:d", look); //$NON-NLS-1$

        try {
            // calculate row height 72dpi based
            final int rowHeigh = this.calculateRowHeight(printer);

            // get copies of FontData for the case we need to scale them
            final FontData indexFontTmp = FontDescriptor.copy(this.indexFont.getFontData()[0]);
            final FontData contentFontTmp = FontDescriptor.copy(this.contentFont.getFontData()[0]);

            // calculate the font height
            final int indexFontHeight = (int) (indexFontTmp.getHeight() * PrintRegister.FACTOR);
            final int regFontHeight = (int) (contentFontTmp.getHeight() * PrintRegister.FACTOR);
            int height = Math.max(indexFontHeight, regFontHeight);
            // add 10 for row spacing and margins
            height += 10;

            // Check if row is too high
            if (height > rowHeigh) {
                // Remove 4 for row spacing
                height = rowHeigh - 4;
                // calculate scaled font height
                final int scaleFontHeight = (int) ((height - 10) / PrintRegister.FACTOR) - 1;
                // set the height to temporary FontData
                indexFontTmp.setHeight(scaleFontHeight);
                contentFontTmp.setHeight(scaleFontHeight);
                Logger.debug("New Font Height: " + indexFontTmp.getHeight()); //$NON-NLS-1$

                // Inform user about scaling
                final String msg = I18N.tr("PrintRegister.fontScalled"); //$NON-NLS-1$
                final StatusBarMessage barMsg = new StatusBarMessage(msg, StatusBarMessage.TYPE_INFO);
                Application.getMessagingFactory().sendMessage(barMsg);

            }

            // Create a EmptyPrint in row height to be re-used
            final EmptyPrint rowHeight = new EmptyPrint(0, height);

            // Index graphics two pixel smaller so vertical alignment looks
            // better
            final ImageData indexData = this.getImage(height - 2, PrintRegister.INDEX_WIDTH, this.indexBgColor);
            final ImagePrint indexPrint = new ImagePrint(indexData, PrintRegister.DPI_72);
            // description graphics two pixel smaller so vertical alignment
            // looks better
            final ImageData descriptionData = this.getImage(height - 2, PrintRegister.DESCRIPTION_WIDTH,
                    this.contentBgColor);
            final ImagePrint descriptionPrint = new ImagePrint(descriptionData, PrintRegister.DPI_72);

            while (this.index.hasNext()) {
                // get the register row
                final Register register = this.index.next();
                // and add them to the grid
                this.addRow(grid, register, rowHeight, indexFontTmp, contentFontTmp, indexPrint, descriptionPrint);
            }
        } catch (final RemoteException e) {
            throw new ApplicationException(I18N.tr("PrintRegister.errorCreatingPrintContent", this.title), //$NON-NLS-1$
                    e);
        }
        return grid;
    }

    /**
     * Adds a row of the register to the given grid.
     *
     * @param grid
     *            to add the row to
     * @param register
     *            the register row to add
     * @param rowHeight
     *            the EmpatyPrintn for the spacing the row high
     * @param indexFontTmp
     *            the font for the index
     * @param contentFontTmp
     *            the font for the content
     * @param indexPrint
     *            the index background graphic
     * @param descriptionPrint
     *            the description background graphic
     * @throws RemoteException
     *             if the register data coudl not be read
     */
    private void addRow(final GridPrint grid, final Register register, final EmptyPrint rowHeight,
            final FontData indexFontTmp, final FontData contentFontTmp, final ImagePrint indexPrint,
            final ImagePrint descriptionPrint) throws RemoteException {

        // Index layer to keep background and text
        LayerPrint layerPrint = new LayerPrint(); // NOPMD
        // The TextPrint to keep the index
        TextPrint textPrint = new TextPrint(register.getIndex(), indexFontTmp); // NOPMD
        // A grid to ensure row height and to be able to vertically
        // center the TextPrint
        final GridPrint textIndex = new GridPrint("4mm, r:d"); //$NON-NLS-1$ //NOPMD
        // Add the EmptyPrint and TextPrint to grid
        textIndex.add(rowHeight);
        textIndex.add(SWT.LEFT, SWT.CENTER, textPrint);
        // add the index background and row grid to the layer
        layerPrint.add(indexPrint);
        layerPrint.add(textIndex);
        // Add the to the final grid
        grid.add(layerPrint);

        // Description layer to keep background and text
        layerPrint = new LayerPrint(); // NOPMD
        // Potentially shorten the description text to fit in one row
        final String description = this.fitDescription(register.getBeschreibung(), contentFontTmp);
        // The TextPrint to keep the index
        textPrint = new TextPrint(description, contentFontTmp); // NOPMD
        // A grid to ensure row height and to be able to vertically
        // center the TextPrint
        final GridPrint textContent = new GridPrint("5mm, r:d"); //$NON-NLS-1$ //NOPMD
        // Add the EmptyPrint and TextPrint to grid
        textContent.add(rowHeight);
        textContent.add(SWT.LEFT, SWT.CENTER, textPrint);
        // add the description background and row grid to the layer
        layerPrint.add(descriptionPrint);
        layerPrint.add(textContent);
        // Add the to the final grid
        grid.add(layerPrint);

    }

    /**
     * Calculates the height of one row if the index as 72dpi based integer
     *
     * @param printer
     * @return the height as 72dpi based integer
     * @throws RemoteException
     */
    private int calculateRowHeight(final Printer printer) throws RemoteException {

        // calculate the 72 dpi factor based on printer resolution
        final int printerFactor = (int) (printer.getDPI().x / 72.0);

        // Initial row height is 0
        int rowHeight = 0;

        // get font height
        final int tinyHheight = AbstractPrint.FONT_TINY.getHeight();
        final int titleHeight = this.titleFont.getFontData()[0].getHeight();

        // get number of tile rows
        final int titelLines = this.title.replaceAll("\r\n", "\n").split("\n").length + 1; //$NON-NLS-1$//$NON-NLS-2$ //$NON-NLS-3$

        // get the total height of header and footer
        final int totalHeight = (int) ((4 * tinyHheight + titelLines * titleHeight
                + 2 * AbstractPrint.HEADER_FOOTER_GAP) * PrintRegister.FACTOR);

        // Calculate print height to 72dpi based
        final int printHeight = PaperClips.getPrintableBounds(printer).height / printerFactor;

        // 1.5 cm margins on top on bottom
        final int margins = Utils.mmToPoints(30);

        // now we can calculate the max. height of one row in the remaining
        // print area
        rowHeight = (printHeight - totalHeight - margins) / this.index.size();

        // and return the result
        return rowHeight;
    }

    /**
     * Checks and evtl shorten the description text to fit into given size
     *
     * @param beschreibung
     * @param contentFontTmp
     * @return evtl shortened description text
     */
    private String fitDescription(final String beschreibung, final FontData contentFontTmp) {

        final StringBuilder shortend = new StringBuilder(beschreibung);
        final GC graphic = new GC(GUI.getShell());
        final Font font = FontDescriptor.createFrom(contentFontTmp).createFont(graphic.getDevice());
        graphic.setFont(font);
        final int descWidth = graphic.textExtent(shortend.toString()).x;
        if (PrintRegister.DESCRIPTION_WIDTH < descWidth) {
            shortend.delete(shortend.length() - 3, shortend.length()).append("..."); //$NON-NLS-1$
            while (PrintRegister.DESCRIPTION_WIDTH < graphic.textExtent(shortend.toString()).x) {
                shortend.deleteCharAt(shortend.length() - 4);
            }
            // shortend = shortend.append("..."); //$NON-NLS-1$
            Logger.debug("Short text  : " + shortend + "(" + graphic.textExtent(shortend.toString()) + ")"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
        }
        font.dispose();
        graphic.dispose();
        return shortend.toString();
    }

    /**
     * Returns the background image for a row with the given height, width and colour
     *
     * @param height
     * @param width
     * @param color
     * @return the row background image
     */
    private ImageData getImage(final int height, final int width, final Color color) {

        final Image image = new Image(GUI.getDisplay(), width, height);
        final GC graphic = new GC(image);
        graphic.setBackground(color);
        graphic.fillOval(0, 0, height / 2, height);
        graphic.fillRectangle(height / 4, 0, width, height);
        graphic.dispose();
        return image.getImageData();
    }

    private String getLastSaveDate() {

        final SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$
        return sdf.format(this.lastSaveDate);
    }
}
